@@ -23,26 +23,7 @@ import os
 import sys
 
 import eventlet
-# NOTE(jokke): As per the eventlet commit
-# b756447bab51046dfc6f1e0e299cc997ab343701 there's circular import happening
-# which can be solved making sure the hubs are properly and fully imported
-# before calling monkey_patch(). This is solved in eventlet 0.22.0 but we
-# need to address it before that is widely used around.
-eventlet.hubs.get_hub()
-
-if os.name == 'nt':
-    # eventlet monkey patching the os module causes subprocess.Popen to fail
-    # on Windows when using pipes due to missing non-blocking IO support.
-    eventlet.patcher.monkey_patch(os=False)
-else:
-    eventlet.patcher.monkey_patch()
-
-# Monkey patch the original current_thread to use the up-to-date _active
-# global variable. See https://bugs.launchpad.net/bugs/1863021 and
-# https://github.com/eventlet/eventlet/issues/592
-import __original_module_threading as orig_threading
-import threading
-orig_threading.current_thread.__globals__['_active'] = threading._active
+eventlet.patcher.monkey_patch()
 
 import subprocess
 
@@ -55,7 +36,6 @@ if os.path.exists(os.path.join(possible_topdir, 'glance', '__init__.py')):
     sys.path.insert(0, possible_topdir)
 
 import glance_store
-from os_win import utilsfactory as os_win_utilsfactory
 from oslo_config import cfg
 from oslo_log import log as logging
 
@@ -70,18 +50,7 @@ CONF.set_default(name='use_stderr', default=True)
 
 
 def main():
-    # Used on Window, ensuring that a single scrubber can run at a time.
-    mutex = None
-    mutex_acquired = False
-
     try:
-        if os.name == 'nt':
-            # We can't rely on process names on Windows as there may be
-            # wrappers with the same name.
-            mutex = os_win_utilsfactory.get_mutex(
-                name='Global\\glance-scrubber')
-            mutex_acquired = mutex.acquire(timeout_ms=0)
-
         CONF.register_cli_opts(scrubber.scrubber_cmd_cli_opts)
         CONF.register_opts(scrubber.scrubber_cmd_opts)
 
@@ -105,12 +74,7 @@ def main():
         app = scrubber.Scrubber(glance_store)
 
         if CONF.restore:
-            if os.name == 'nt':
-                scrubber_already_running = not mutex_acquired
-            else:
-                scrubber_already_running = scrubber_already_running_posix()
-
-            if scrubber_already_running:
+            if scrubber_already_running():
                 already_running_msg = (
                     "ERROR: glance-scrubber is already running. "
                     "Please ensure that the daemon is stopped.")
@@ -127,12 +91,9 @@ def main():
         sys.exit("ERROR: %s" % e)
     except RuntimeError as e:
         sys.exit("ERROR: %s" % e)
-    finally:
-        if mutex and mutex_acquired:
-            mutex.release()
 
 
-def scrubber_already_running_posix():
+def scrubber_already_running():
     # Try to check the glance-scrubber is running or not.
     # 1. Try to find the pid file if scrubber is controlled by
     #    glance-control
