@@ -25,9 +25,9 @@ from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_utils import encodeutils
 from oslo_utils import excutils
+from oslo_utils.imageutils import format_inspector
 
 from glance.common import exception
-from glance.common import format_inspector
 from glance.common import store_utils
 from glance.common import utils
 import glance.domain.proxy
@@ -580,37 +580,88 @@ class ImageProxy(glance.domain.proxy.Image):
                 img_signature_key_type=key_type
             )
 
-        if not self.image.virtual_size:
-            inspector = format_inspector.get_inspector(self.image.disk_format)
+        if self.image.container_format == 'bare':
+            # FIXME(danms): We do not pass an expected_format here because
+            # we do not (currently) want to interrupt the data pipeline if
+            # the format does not match.
+            LOG.debug('Enabling in-flight format inspection for %s',
+                      self.image.disk_format)
+            data = format_inspector.InspectWrapper(data)
         else:
-            # No need to do this again
-            inspector = None
+            # FIXME(danms): Either warn here if we are unable to inspect non-
+            # bare images or maybe make this a hard fail
+            pass
 
-        if inspector and self.image.container_format == 'bare':
-            fmt = inspector()
-            data = format_inspector.InfoWrapper(data, fmt)
-            LOG.debug('Enabling in-flight format inspection for %s', fmt)
-        else:
-            fmt = None
-
-        self._upload_to_store(data, verifier, backend, size)
+        try:
+            self._upload_to_store(data, verifier, backend, size)
+        except format_inspector.ImageFormatError as e:
+            raise exception.InvalidImageData(str(e))
+        finally:
+            try:
+                data.close()
+            except AttributeError:
+                # We did not get a closeable or file-like object as data and/or
+                # did not wrap it because of container_format
+                pass
 
         virtual_size = 0
-        if fmt and fmt.format_match:
-            try:
-                virtual_size = fmt.virtual_size
+        try:
+            inspector = data.format
+            format = str(inspector)
+            if format == self.image.disk_format:
+                virtual_size = inspector.virtual_size
                 LOG.info('Image format matched and virtual size computed: %i',
                          virtual_size)
-            except Exception as e:
-                LOG.error(_LE('Unable to determine virtual_size because: %s'),
-                          e)
-        elif fmt:
-            LOG.warning('Image format %s did not match; '
-                        'unable to calculate virtual size',
-                        self.image.disk_format)
+            elif (format == 'raw' and
+                  self.image.disk_format not in format_inspector.ALL_FORMATS):
+                # Things like AMI are not inspectable, so as long as we did not
+                # recognize the format of the payload, allow it through.
+                LOG.info('Image format %s is not an inspectable type',
+                         self.image.disk_format)
+            elif self.image.disk_format == 'raw':
+                # This is something we can't inspect, so we allow it
+                LOG.warning('Image claims %r disk_format but is actually %r',
+                            data.format, format)
+                # If we are pretending this is really raw, don't use the
+                # detected rich format's virtual_size, but just use the
+                # actual size in bytes, as if we did not recognize the format.
+                virtual_size = inspector.actual_size
+            else:
+                raise format_inspector.ImageFormatError(
+                    _('Image disk_format %s does not match stream data %s' % (
+                        self.image.disk_format, format)))
+            inspector.safety_check()
+        except AttributeError:
+            # We must not have wrapped this for inspection because of
+            # container_format
+            pass
+        except format_inspector.ImageFormatError as e:
+            # No format matched!
+            if CONF.image_format.require_image_format_match:
+                raise exception.InvalidImageData(str(e))
+            else:
+                LOG.warning('Image format %s did not match; '
+                            'unable to calculate virtual size',
+                            self.image.disk_format)
+        except format_inspector.SafetyCheckFailed as e:
+            nonfatal = set(CONF.image_format.gpt_safety_checks_nonfatal)
+            fatal = e.failures.keys() - nonfatal
+            if inspector.NAME == 'gpt' and not fatal:
+                # All the failures were non-fatal (per config) so log them and
+                # continue
+                LOG.warning('Non-fatal %s', e)
+            else:
+                LOG.warning('Image %s %s', format, e)
+                raise exception.InvalidImageData('Image failed safety checks')
+        except Exception as e:
+            LOG.error(_LE('Unable to determine stream format because: %s'), e)
+            # FIXME(danms): Do this based on config
+            if CONF.image_format.require_image_format_match:
+                raise exception.InvalidImageData(
+                    _('Image format inspection failed'))
 
         if virtual_size:
-            self.image.virtual_size = fmt.virtual_size
+            self.image.virtual_size = virtual_size
 
         if set_active and self.image.status != 'active':
             self.image.status = 'active'
