@@ -13,6 +13,8 @@
 import itertools
 
 from glance.policies import base
+from glance.policies import cache
+from glance.policies import discovery
 from glance.policies import image
 from glance.policies import metadef
 from glance.policies import tasks
@@ -24,4 +26,6 @@ def list_rules():
         image.list_rules(),
         tasks.list_rules(),
         metadef.list_rules(),
+        cache.list_rules(),
+        discovery.list_rules(),
     )
