@@ -30,7 +30,6 @@ import platform
 import shutil
 import signal
 import socket
-import subprocess
 import sys
 import tempfile
 from testtools import content as ttc
@@ -42,7 +41,6 @@ import uuid
 
 import fixtures
 import glance_store
-from os_win import utilsfactory as os_win_utilsfactory
 from oslo_config import cfg
 from oslo_serialization import jsonutils
 import testtools
@@ -58,12 +56,7 @@ from glance.tests import utils as test_utils
 execute, get_unused_port = test_utils.execute, test_utils.get_unused_port
 tracecmd_osmap = {'Linux': 'strace', 'FreeBSD': 'truss'}
 
-if os.name == 'nt':
-    SQLITE_CONN_TEMPLATE = 'sqlite:///%s/tests.sqlite'
-else:
-    SQLITE_CONN_TEMPLATE = 'sqlite:////%s/tests.sqlite'
-
-
+SQLITE_CONN_TEMPLATE = 'sqlite:////%s/tests.sqlite'
 CONF = cfg.CONF
 
 
@@ -99,6 +92,8 @@ class BaseServer(metaclass=abc.ABCMeta):
         self.deployment_flavor = ''
         self.show_image_direct_url = False
         self.show_multiple_locations = False
+        self.do_secure_hash = True
+        self.http_retries = '3'
         self.property_protection_file = ''
         self.needs_database = False
         self.log_file = None
@@ -107,7 +102,6 @@ class BaseServer(metaclass=abc.ABCMeta):
         self.process_pid = None
         self.server_module = None
         self.stop_kill = False
-        self.send_identity_credentials = False
 
     def write_conf(self, **kwargs):
         """
@@ -181,8 +175,8 @@ class BaseServer(metaclass=abc.ABCMeta):
             conf_filepath = os.path.join(conf_dir, 'glance-manage.conf')
 
             with open(conf_filepath, 'w') as conf_file:
-                conf_file.write('[DEFAULT]\n')
-                conf_file.write('sql_connection = %s' % self.sql_connection)
+                conf_file.write('[database]\n')
+                conf_file.write('connection = %s' % self.sql_connection)
                 conf_file.flush()
 
             glance_db_env = 'GLANCE_DB_TEST_SQLITE_FILE'
@@ -300,79 +294,7 @@ class PosixServer(BaseServer):
         return (rc, '', '')
 
 
-class Win32Server(BaseServer):
-    def __init__(self, *args, **kwargs):
-        super(Win32Server, self).__init__(*args, **kwargs)
-
-        self._processutils = os_win_utilsfactory.get_processutils()
-
-    def start(self, expect_exit=True, expected_exitcode=0, **kwargs):
-        """
-        Starts the server.
-
-        Any kwargs passed to this method will override the configuration
-        value in the conf file used in starting the servers.
-        """
-
-        # Ensure the configuration file is written
-        self.write_conf(**kwargs)
-
-        self.create_database()
-
-        cmd = ("%(server_module)s --config-file %(conf_file_name)s"
-               % {"server_module": self.server_module,
-                  "conf_file_name": self.conf_file_name})
-        cmd = "%s -m %s" % (sys.executable, cmd)
-
-        # Passing socket objects on Windows is a bit more cumbersome.
-        # We don't really have to do it.
-        if self.sock:
-            self.sock.close()
-            self.sock = None
-
-        self.process = subprocess.Popen(
-            cmd,
-            env=self.exec_env)
-        self.process_pid = self.process.pid
-
-        try:
-            self.job_handle = self._processutils.kill_process_on_job_close(
-                self.process_pid)
-        except Exception:
-            # Could not associate child process with a job, killing it.
-            self.process.kill()
-            raise
-
-        self.stop_kill = not expect_exit
-        if self.pid_file:
-            pf = open(self.pid_file, 'w')
-            pf.write('%d\n' % self.process_pid)
-            pf.close()
-
-        rc = 0
-        if expect_exit:
-            self.process.communicate()
-            rc = self.process.returncode
-
-        return (rc, '', '')
-
-    def stop(self):
-        """
-        Spin down the server.
-        """
-        if not self.process_pid:
-            raise Exception('Server "%s" process not running.'
-                            % self.server_name)
-
-        if self.stop_kill:
-            self.process.terminate()
-        return (0, '', '')
-
-
-if os.name == 'nt':
-    Server = Win32Server
-else:
-    Server = PosixServer
+Server = PosixServer
 
 
 class ApiServer(Server):
@@ -408,9 +330,7 @@ class ApiServer(Server):
         self.image_location_quota = 2
         self.disable_path = None
 
-        secure_rbac = bool(os.getenv('OS_GLANCE_TEST_RBAC_DEFAULTS'))
-        self.enforce_secure_rbac = secure_rbac
-        self.enforce_new_defaults = secure_rbac
+        self.enforce_new_defaults = True
 
         self.needs_database = True
         default_sql_connection = SQLITE_CONN_TEMPLATE % self.test_dir
@@ -419,11 +339,6 @@ class ApiServer(Server):
         self.user_storage_quota = '0'
         self.lock_path = self.test_dir
 
-        self.location_strategy = 'location_order'
-        self.store_type_location_strategy_preference = ""
-
-        self.send_identity_headers = False
-
         self.node_staging_uri = 'file://%s' % os.path.join(
             self.test_dir, 'staging')
 
@@ -433,18 +348,17 @@ default_log_levels = eventlet.wsgi.server=DEBUG,stevedore.extension=INFO
 bind_host = %(bind_host)s
 bind_port = %(bind_port)s
 metadata_encryption_key = %(metadata_encryption_key)s
-send_identity_credentials = %(send_identity_credentials)s
 log_file = %(log_file)s
 image_size_cap = %(image_size_cap)d
 delayed_delete = %(delayed_delete)s
 workers = %(workers)s
 scrub_time = %(scrub_time)s
-send_identity_headers = %(send_identity_headers)s
 image_cache_dir = %(image_cache_dir)s
 image_cache_driver = %(image_cache_driver)s
-sql_connection = %(sql_connection)s
 show_image_direct_url = %(show_image_direct_url)s
 show_multiple_locations = %(show_multiple_locations)s
+do_secure_hash = %(do_secure_hash)s
+http_retries = %(http_retries)s
 user_storage_quota = %(user_storage_quota)s
 lock_path = %(lock_path)s
 property_protection_file = %(property_protection_file)s
@@ -453,41 +367,50 @@ image_member_quota=%(image_member_quota)s
 image_property_quota=%(image_property_quota)s
 image_tag_quota=%(image_tag_quota)s
 image_location_quota=%(image_location_quota)s
-location_strategy=%(location_strategy)s
-allow_additional_image_properties = True
 node_staging_uri=%(node_staging_uri)s
-enforce_secure_rbac=%(enforce_secure_rbac)s
+[database]
+connection = %(sql_connection)s
 [oslo_policy]
 policy_file = %(policy_file)s
 policy_default_rule = %(policy_default_rule)s
 enforce_new_defaults=%(enforce_new_defaults)s
 [paste_deploy]
 flavor = %(deployment_flavor)s
-[store_type_location_strategy]
-store_type_preference = %(store_type_location_strategy_preference)s
 [glance_store]
 filesystem_store_datadir=%(image_dir)s
 default_store = %(default_store)s
 [import_filtering_opts]
 allowed_ports = []
 """
-        self.paste_conf_base = """[pipeline:glance-api]
+        self.paste_conf_base = """[composite:glance-api]
+paste.composite_factory = glance.api:root_app_factory
+/: api
+/healthcheck: healthcheck
+
+[pipeline:api]
 pipeline =
     cors
-    healthcheck
     versionnegotiation
     gzip
     unauthenticated-context
     rootapp
 
-[pipeline:glance-api-caching]
-pipeline = cors healthcheck versionnegotiation gzip context
- cache rootapp
+[composite:glance-api-caching]
+paste.composite_factory = glance.api:root_app_factory
+/: api-caching
+/healthcheck: healthcheck
 
-[pipeline:glance-api-cachemanagement]
+[pipeline:api-caching]
+pipeline = cors versionnegotiation gzip context cache rootapp
+
+[composite:glance-api-cachemanagement]
+paste.composite_factory = glance.api:root_app_factory
+/: api-cachemanagement
+/healthcheck: healthcheck
+
+[pipeline:api-cachemanagement]
 pipeline =
     cors
-    healthcheck
     versionnegotiation
     gzip
     unauthenticated-context
@@ -495,11 +418,21 @@ pipeline =
     cache_manage
     rootapp
 
-[pipeline:glance-api-fakeauth]
-pipeline = cors healthcheck versionnegotiation gzip fakeauth context rootapp
+[composite:glance-api-fakeauth]
+paste.composite_factory = glance.api:root_app_factory
+/: api-fakeauth
+/healthcheck: healthcheck
 
-[pipeline:glance-api-noauth]
-pipeline = cors healthcheck versionnegotiation gzip context rootapp
+[pipeline:api-fakeauth]
+pipeline = cors versionnegotiation gzip fakeauth context rootapp
+
+[composite:glance-api-noauth]
+paste.composite_factory = glance.api:root_app_factory
+/: api-noauth
+/healthcheck: healthcheck
+
+[pipeline:api-noauth]
+pipeline = cors versionnegotiation gzip context rootapp
 
 [composite:rootapp]
 paste.composite_factory = glance.api:root_app_factory
@@ -512,14 +445,13 @@ paste.app_factory = glance.api.versions:create_resource
 [app:apiv2app]
 paste.app_factory = glance.api.v2.router:API.factory
 
-[filter:healthcheck]
-paste.filter_factory = oslo_middleware:Healthcheck.factory
+[app:healthcheck]
+paste.app_factory = oslo_middleware:Healthcheck.app_factory
 backends = disable_by_file
 disable_by_file_path = %(disable_path)s
 
 [filter:versionnegotiation]
-paste.filter_factory =
- glance.api.middleware.version_negotiation:VersionNegotiationFilter.factory
+paste.filter_factory = glance.api.middleware.version_negotiation:VersionNegotiationFilter.factory
 
 [filter:gzip]
 paste.filter_factory = glance.api.middleware.gzip:GzipMiddleware.factory
@@ -528,15 +460,13 @@ paste.filter_factory = glance.api.middleware.gzip:GzipMiddleware.factory
 paste.filter_factory = glance.api.middleware.cache:CacheFilter.factory
 
 [filter:cache_manage]
-paste.filter_factory =
- glance.api.middleware.cache_manage:CacheManageFilter.factory
+paste.filter_factory = glance.api.middleware.cache_manage:CacheManageFilter.factory
 
 [filter:context]
 paste.filter_factory = glance.api.middleware.context:ContextMiddleware.factory
 
 [filter:unauthenticated-context]
-paste.filter_factory =
- glance.api.middleware.context:UnauthenticatedContextMiddleware.factory
+paste.filter_factory = glance.api.middleware.context:UnauthenticatedContextMiddleware.factory
 
 [filter:fakeauth]
 paste.filter_factory = glance.tests.utils:FakeAuthMiddleware.factory
@@ -585,6 +515,8 @@ class ApiServerForMultipleBackend(Server):
         self.image_location_quota = 2
         self.disable_path = None
 
+        self.enforce_new_defaults = True
+
         self.needs_database = True
         default_sql_connection = SQLITE_CONN_TEMPLATE % self.test_dir
         self.sql_connection = os.environ.get('GLANCE_TEST_SQL_CONNECTION',
@@ -592,29 +524,23 @@ class ApiServerForMultipleBackend(Server):
         self.user_storage_quota = '0'
         self.lock_path = self.test_dir
 
-        self.location_strategy = 'location_order'
-        self.store_type_location_strategy_preference = ""
-
-        self.send_identity_headers = False
-
         self.conf_base = """[DEFAULT]
 debug = %(debug)s
 default_log_levels = eventlet.wsgi.server=DEBUG,stevedore.extension=INFO
 bind_host = %(bind_host)s
 bind_port = %(bind_port)s
 metadata_encryption_key = %(metadata_encryption_key)s
-send_identity_credentials = %(send_identity_credentials)s
 log_file = %(log_file)s
 image_size_cap = %(image_size_cap)d
 delayed_delete = %(delayed_delete)s
 workers = %(workers)s
 scrub_time = %(scrub_time)s
-send_identity_headers = %(send_identity_headers)s
 image_cache_dir = %(image_cache_dir)s
 image_cache_driver = %(image_cache_driver)s
-sql_connection = %(sql_connection)s
 show_image_direct_url = %(show_image_direct_url)s
 show_multiple_locations = %(show_multiple_locations)s
+do_secure_hash = %(do_secure_hash)s
+http_retries = %(http_retries)s
 user_storage_quota = %(user_storage_quota)s
 lock_path = %(lock_path)s
 property_protection_file = %(property_protection_file)s
@@ -623,16 +549,15 @@ image_member_quota=%(image_member_quota)s
 image_property_quota=%(image_property_quota)s
 image_tag_quota=%(image_tag_quota)s
 image_location_quota=%(image_location_quota)s
-location_strategy=%(location_strategy)s
-allow_additional_image_properties = True
 enabled_backends=file1:file,file2:file,file3:file
+[database]
+connection = %(sql_connection)s
 [oslo_policy]
 policy_file = %(policy_file)s
 policy_default_rule = %(policy_default_rule)s
+enforce_new_defaults=%(enforce_new_defaults)s
 [paste_deploy]
 flavor = %(deployment_flavor)s
-[store_type_location_strategy]
-store_type_preference = %(store_type_location_strategy_preference)s
 [glance_store]
 default_backend = %(default_backend)s
 [file1]
@@ -646,23 +571,35 @@ allowed_ports = []
 [os_glance_staging_store]
 filesystem_store_datadir=%(staging_dir)s
 """
-        self.paste_conf_base = """[pipeline:glance-api]
+        self.paste_conf_base = """[composite:glance-api]
+paste.composite_factory = glance.api:root_app_factory
+/: api
+/healthcheck: healthcheck
+
+[pipeline:api]
 pipeline =
     cors
-    healthcheck
     versionnegotiation
     gzip
     unauthenticated-context
     rootapp
 
-[pipeline:glance-api-caching]
-pipeline = cors healthcheck versionnegotiation gzip unauthenticated-context
- cache rootapp
+[composite:glance-api-caching]
+paste.composite_factory = glance.api:root_app_factory
+/: api-caching
+/healthcheck: healthcheck
 
-[pipeline:glance-api-cachemanagement]
+[pipeline:api-caching]
+pipeline = cors versionnegotiation gzip unauthenticated-context cache rootapp
+
+[composite:glance-api-cachemanagement]
+paste.composite_factory = glance.api:root_app_factory
+/: api-cachemanagement
+/healthcheck: healthcheck
+
+[pipeline:api-cachemanagement]
 pipeline =
     cors
-    healthcheck
     versionnegotiation
     gzip
     unauthenticated-context
@@ -670,11 +607,21 @@ pipeline =
     cache_manage
     rootapp
 
-[pipeline:glance-api-fakeauth]
-pipeline = cors healthcheck versionnegotiation gzip fakeauth context rootapp
+[composite:glance-api-fakeauth]
+paste.composite_factory = glance.api:root_app_factory
+/: api-fakeauth
+/healthcheck: healthcheck
 
-[pipeline:glance-api-noauth]
-pipeline = cors healthcheck versionnegotiation gzip context rootapp
+[pipeline:api-fakeauth]
+pipeline = cors versionnegotiation gzip fakeauth context rootapp
+
+[composite:glance-api-noauth]
+paste.composite_factory = glance.api:root_app_factory
+/: api-noauth
+/healthcheck: healthcheck
+
+[pipeline:api-noauth]
+pipeline = cors versionnegotiation gzip context rootapp
 
 [composite:rootapp]
 paste.composite_factory = glance.api:root_app_factory
@@ -687,14 +634,13 @@ paste.app_factory = glance.api.versions:create_resource
 [app:apiv2app]
 paste.app_factory = glance.api.v2.router:API.factory
 
-[filter:healthcheck]
-paste.filter_factory = oslo_middleware:Healthcheck.factory
+[app:healthcheck]
+paste.app_factory = oslo_middleware:Healthcheck.app_factory
 backends = disable_by_file
 disable_by_file_path = %(disable_path)s
 
 [filter:versionnegotiation]
-paste.filter_factory =
- glance.api.middleware.version_negotiation:VersionNegotiationFilter.factory
+paste.filter_factory = glance.api.middleware.version_negotiation:VersionNegotiationFilter.factory
 
 [filter:gzip]
 paste.filter_factory = glance.api.middleware.gzip:GzipMiddleware.factory
@@ -703,15 +649,13 @@ paste.filter_factory = glance.api.middleware.gzip:GzipMiddleware.factory
 paste.filter_factory = glance.api.middleware.cache:CacheFilter.factory
 
 [filter:cache_manage]
-paste.filter_factory =
- glance.api.middleware.cache_manage:CacheManageFilter.factory
+paste.filter_factory = glance.api.middleware.cache_manage:CacheManageFilter.factory
 
 [filter:context]
 paste.filter_factory = glance.api.middleware.context:ContextMiddleware.factory
 
 [filter:unauthenticated-context]
-paste.filter_factory =
- glance.api.middleware.context:UnauthenticatedContextMiddleware.factory
+paste.filter_factory = glance.api.middleware.context:UnauthenticatedContextMiddleware.factory
 
 [filter:fakeauth]
 paste.filter_factory = glance.tests.utils:FakeAuthMiddleware.factory
@@ -747,8 +691,6 @@ class ScrubberDaemon(Server):
         self.policy_file = policy_file
         self.policy_default_rule = 'default'
 
-        self.send_identity_headers = False
-
         self.conf_base = """[DEFAULT]
 debug = %(debug)s
 log_file = %(log_file)s
@@ -757,8 +699,9 @@ wakeup_time = 2
 scrub_time = %(scrub_time)s
 metadata_encryption_key = %(metadata_encryption_key)s
 lock_path = %(lock_path)s
-sql_connection = %(sql_connection)s
 sql_idle_timeout = 3600
+[database]
+connection = %(sql_connection)s
 [glance_store]
 filesystem_store_datadir=%(image_dir)s
 [oslo_policy]
@@ -800,7 +743,7 @@ class FunctionalTest(test_utils.BaseTestCase):
         # The clients will try to connect to this address. Let's make sure
         # we're not using the default '0.0.0.0'
         self.config(bind_host='127.0.0.1')
-
+        self.config(image_cache_dir=self.test_dir)
         self.tracecmd = tracecmd_osmap.get(platform.system())
 
         conf_dir = os.path.join(self.test_dir, 'etc')
@@ -1104,15 +1047,6 @@ class FunctionalTest(test_utils.BaseTestCase):
         if self.include_scrubber:
             self.stop_server(self.scrubber_daemon)
 
-    def run_sql_cmd(self, sql):
-        """
-        Provides a crude mechanism to run manual SQL commands for backend
-        DB verification within the functional tests.
-        The raw result set is returned.
-        """
-        engine = db_api.get_engine()
-        return engine.execute(sql)
-
     def copy_data_file(self, file_name, dst_dir):
         src_file_name = os.path.join('glance/tests/etc', file_name)
         shutil.copy(src_file_name, dst_dir)
@@ -1456,15 +1390,6 @@ class MultipleBackendFunctionalTest(test_utils.BaseTestCase):
         if self.include_scrubber:
             self.stop_server(self.scrubber_daemon)
 
-    def run_sql_cmd(self, sql):
-        """
-        Provides a crude mechanism to run manual SQL commands for backend
-        DB verification within the functional tests.
-        The raw result set is returned.
-        """
-        engine = db_api.get_engine()
-        return engine.execute(sql)
-
     def copy_data_file(self, file_name, dst_dir):
         src_file_name = os.path.join('glance/tests/etc', file_name)
         shutil.copy(src_file_name, dst_dir)
@@ -1600,7 +1525,7 @@ class SynchronousAPIBase(test_utils.BaseTestCase):
         self.setup_simple_paste()
         self.setup_stores()
 
-    def start_server(self, enable_cache=True):
+    def start_server(self, enable_cache=True, set_worker_url=True):
         """Builds and "starts" the API server.
 
         Note that this doesn't actually "start" anything like
@@ -1613,11 +1538,14 @@ class SynchronousAPIBase(test_utils.BaseTestCase):
             root_app = 'glance-api-cachemanagement'
             self.config(image_cache_dir=self._store_dir('cache'))
 
+        if set_worker_url:
+            self.config(worker_self_reference_url='http://workerx')
+
         self.api = config.load_paste_app(root_app,
                                          conf_file=self.paste_config)
-        secure_rbac = bool(os.getenv('OS_GLANCE_TEST_RBAC_DEFAULTS'))
-        self.config(enforce_secure_rbac=secure_rbac)
-        self.config(enforce_new_defaults=secure_rbac,
+        self.config(enforce_new_defaults=True,
+                    group='oslo_policy')
+        self.config(enforce_scope=True,
                     group='oslo_policy')
 
     def _headers(self, custom_headers=None):
