@@ -12,10 +12,14 @@
 #    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 #    License for the specific language governing permissions and limitations
 #    under the License.
+import io
+
 from cryptography import exceptions as crypto_exception
 from cursive import exception as cursive_exception
 from cursive import signature_utils
 import glance_store
+from oslo_config import cfg
+import struct
 from unittest import mock
 
 from glance.common import exception
@@ -32,6 +36,7 @@ USER1 = '54492ba0-f4df-4e4e-be62-27f4d76b29cf'
 TENANT1 = '6838eb7b-6ded-434a-882c-b344c77fe8df'
 TENANT2 = '2c014f32-55eb-467d-8fcb-4bd706012f81'
 TENANT3 = '228c6da5-29cd-4d67-9457-ed632e083fc0'
+CONF = cfg.CONF
 
 
 class ImageRepoStub(object):
@@ -114,17 +119,26 @@ class TestStoreMultiBackends(utils.BaseTestCase):
         image_stub = ImageStub(UUID2, status='queued', locations=[],
                                extra_properties=extra_properties,
                                virtual_size=4)
-        image_stub.disk_format = 'iso'
+        image_stub.disk_format = 'raw'
         image = glance.location.ImageProxy(image_stub, context,
                                            self.store_api, self.store_utils)
+        found_data = []
+
+        def consumer(data, _v, _b, _s):
+            found_data.append(data.read(4))
+
         with mock.patch.object(image, "_upload_to_store") as mloc:
-            image.set_data('YYYY', 4, backend='ceph1')
+            mloc.side_effect = consumer
+            src_data = io.BytesIO(b'YYYY')
+            image.set_data(src_data, 4, backend='ceph1')
             msig.assert_called_once_with(context=context,
                                          img_signature_certificate_uuid='UUID',
                                          img_signature_hash_method='METHOD',
                                          img_signature='VALID',
                                          img_signature_key_type='TYPE')
-            mloc.assert_called_once_with('YYYY', msig.return_value, 'ceph1', 4)
+            mloc.assert_called_once_with(mock.ANY, msig.return_value, 'ceph1',
+                                         4)
+            self.assertEqual([b'YYYY'], found_data)
 
         self.assertEqual('active', image.status)
 
@@ -274,6 +288,27 @@ class TestStoreImage(utils.BaseTestCase):
         # We are going to pass an iterable data source, so use the
         # FakeStoreAPIReader that actually reads from that data
         store_api = unit_test_utils.FakeStoreAPIReader()
+        image = glance.location.ImageProxy(image_stub, context,
+                                           store_api, self.store_utils)
+        if CONF.image_format.require_image_format_match:
+            self.assertRaises(exception.InvalidImageData,
+                              image.set_data, iter(['YYYY']), 4)
+        else:
+            image.set_data(iter(['YYYY']), 4)
+            self.assertEqual(4, image.size)
+            # NOTE(markwash): FakeStore returns image_id for location
+            self.assertEqual(UUID2, image.locations[0]['url'])
+            self.assertEqual('Z', image.checksum)
+            self.assertEqual('active', image.status)
+            self.assertEqual(0, image.virtual_size)
+
+    def test_image_set_data_inspector_unsupported_format(self):
+        context = glance.context.RequestContext(user=USER1)
+        image_stub = ImageStub(UUID2, status='queued', locations=[])
+        image_stub.disk_format = 'ami'
+        # We are going to pass an iterable data source, so use the
+        # FakeStoreAPIReader that actually reads from that data
+        store_api = unit_test_utils.FakeStoreAPIReader()
         image = glance.location.ImageProxy(image_stub, context,
                                            store_api, self.store_utils)
         image.set_data(iter(['YYYY']), 4)
@@ -284,9 +319,106 @@ class TestStoreImage(utils.BaseTestCase):
         self.assertEqual('active', image.status)
         self.assertEqual(0, image.virtual_size)
 
-    @mock.patch('glance.common.format_inspector.QcowInspector.virtual_size',
+    def _get_stub_gpt(self):
+        """Returns a valid, safe GPT-based image"""
+        data = bytearray(b'\x00' * 1024)
+        # The last two bytes of the first sector is the little-endian signature
+        # value 0xAA55
+        data[510:512] = b'\x55\xAA'
+
+        # This is one EFI Protective MBR partition in the first PTE slot,
+        # which is 16 bytes starting at offset 446.
+        data[446:446 + 16] = struct.pack('<BBBBBBBBII',
+                                         0x00,  # boot
+                                         0x00,  # start C
+                                         0x02,  # start H
+                                         0x00,  # start S
+                                         0xEE,  # OS type
+                                         0x00,  # end C
+                                         0x00,  # end H
+                                         0x00,  # end S
+                                         0x01,  # start LBA
+                                         0x00,  # size LBA
+                                         )
+        return data
+
+    def test_image_set_data_inspector_gpt_as_raw_compat(self):
+        context = glance.context.RequestContext(user=USER1)
+        image_stub = ImageStub(UUID2, status='queued', locations=[])
+        image_stub.disk_format = 'raw'
+        # We are going to pass an iterable data source, so use the
+        # FakeStoreAPIReader that actually reads from that data
+
+        store_api = unit_test_utils.FakeStoreAPIReader(max_size=2048)
+        image = glance.location.ImageProxy(image_stub, context,
+                                           store_api, self.store_utils)
+        data = self._get_stub_gpt()
+        image.set_data(iter((data,)), 1024)
+        self.assertEqual(1024, image.size)
+        # NOTE(markwash): FakeStore returns image_id for location
+        self.assertEqual(UUID2, image.locations[0]['url'])
+        self.assertEqual('Z', image.checksum)
+        self.assertEqual('active', image.status)
+        self.assertEqual(1024, image.virtual_size)
+
+    def test_image_set_data_inspector_format_unsupported_hiding(self):
+        context = glance.context.RequestContext(user=USER1)
+        image_stub = ImageStub(UUID2, status='queued', locations=[])
+        image_stub.disk_format = 'ami'
+        # We are going to pass an iterable data source, so use the
+        # FakeStoreAPIReader that actually reads from that data
+        data = self._get_stub_gpt()
+        store_api = unit_test_utils.FakeStoreAPIReader(max_size=2048)
+        image = glance.location.ImageProxy(image_stub, context,
+                                           store_api, self.store_utils)
+        # Image was claimed to be AMI, which we cannot inspect, but we found
+        # a known format inside, which is invalid
+        self.assertRaises(exception.InvalidImageData,
+                          image.set_data, iter((data,)), 1024)
+
+    def test_image_set_data_inspector_format_fails_safety_check(self):
+        self.config(gpt_safety_checks_nonfatal=[], group='image_format')
+        context = glance.context.RequestContext(user=USER1)
+        image_stub = ImageStub(UUID2, status='queued', locations=[])
+        image_stub.disk_format = 'gpt'
+        # We are going to pass an iterable data source, so use the
+        # FakeStoreAPIReader that actually reads from that data
+        data = self._get_stub_gpt()
+        # Zero out the partition table to make this unsafe
+        data[446:446 + 16] = b'\x00' * 16
+        store_api = unit_test_utils.FakeStoreAPIReader(max_size=2048)
+        image = glance.location.ImageProxy(image_stub, context,
+                                           store_api, self.store_utils)
+        # Image was claimed to be AMI, which we cannot inspect, but we found
+        # a known format inside, which is invalid
+        self.assertRaisesRegex(exception.InvalidImageData,
+                               'safety checks',
+                               image.set_data, iter((data,)), 1024)
+
+    def test_image_set_data_gpt_safety_ignored(self):
+        self.config(gpt_safety_checks_nonfatal=['mbr'], group='image_format')
+        context = glance.context.RequestContext(user=USER1)
+        image_stub = ImageStub(UUID2, status='queued', locations=[])
+        image_stub.disk_format = 'gpt'
+        # We are going to pass an iterable data source, so use the
+        # FakeStoreAPIReader that actually reads from that data
+        data = self._get_stub_gpt()
+        # Zero out the partition table to make this unsafe
+        data[446:446 + 16] = b'\x00' * 16
+        store_api = unit_test_utils.FakeStoreAPIReader(max_size=2048)
+        image = glance.location.ImageProxy(image_stub, context,
+                                           store_api, self.store_utils)
+        image.set_data(iter((data,)), 1024)
+
+    def test_image_set_data_inspector_no_match_disabled(self):
+        self.config(require_image_format_match=False, group='image_format')
+        self.test_image_set_data_inspector_no_match()
+
+    @mock.patch('oslo_utils.imageutils.format_inspector.QcowInspector.'
+                'virtual_size',
                 new_callable=mock.PropertyMock)
-    @mock.patch('glance.common.format_inspector.QcowInspector.format_match',
+    @mock.patch('oslo_utils.imageutils.format_inspector.QcowInspector.'
+                'format_match',
                 new_callable=mock.PropertyMock)
     def test_image_set_data_inspector_virtual_size_failure(self, mock_fm,
                                                            mock_vs):
@@ -305,21 +437,30 @@ class TestStoreImage(utils.BaseTestCase):
         image = glance.location.ImageProxy(image_stub, context,
                                            store_api, self.store_utils)
 
-        # Make sure set_data proceeds even though the format clearly
-        # does not match
-        image.set_data(iter(['YYYY']), 4)
-        self.assertEqual(4, image.size)
-        # NOTE(markwash): FakeStore returns image_id for location
-        self.assertEqual(UUID2, image.locations[0]['url'])
-        self.assertEqual('Z', image.checksum)
-        self.assertEqual('active', image.status)
-        self.assertEqual(0, image.virtual_size)
+        if CONF.image_format.require_image_format_match:
+            self.assertRaises(exception.InvalidImageData,
+                              image.set_data, iter(['YYYY']), 4)
+        else:
+            # Make sure set_data proceeds even though the format clearly
+            # does not match
+            image.set_data(iter(['YYYY']), 4)
+            self.assertEqual(4, image.size)
+            # NOTE(markwash): FakeStore returns image_id for location
+            self.assertEqual(UUID2, image.locations[0]['url'])
+            self.assertEqual('Z', image.checksum)
+            self.assertEqual('active', image.status)
+            self.assertEqual(0, image.virtual_size)
 
-    @mock.patch('glance.common.format_inspector.get_inspector')
-    def test_image_set_data_inspector_not_needed(self, mock_gi):
+    def test_image_set_data_inspector_virtual_size_failure_disabled(self):
+        self.config(require_image_format_match=False, group='image_format')
+        self.test_image_set_data_inspector_virtual_size_failure()
+
+    @mock.patch('oslo_utils.imageutils.format_inspector.InspectWrapper')
+    def test_image_set_data_inspector_not_needed(self, mock_inspect):
         context = glance.context.RequestContext(user=USER1)
         image_stub = ImageStub(UUID2, status='queued', locations=[])
         image_stub.virtual_size = 123
+        image_stub.container_format = 'gzip'
         image_stub.disk_format = 'qcow2'
         # We are going to pass an iterable data source, so use the
         # FakeStoreAPIReader that actually reads from that data
@@ -335,7 +476,7 @@ class TestStoreImage(utils.BaseTestCase):
         self.assertEqual(123, image.virtual_size)
         # If the image already had virtual_size set (i.e. we're setting
         # a new location), we should not re-calculate the value.
-        mock_gi.assert_not_called()
+        mock_inspect.assert_not_called()
 
     def test_image_set_data_location_metadata(self):
         context = glance.context.RequestContext(user=USER1)
@@ -364,17 +505,26 @@ class TestStoreImage(utils.BaseTestCase):
         image_stub.disk_format = 'iso'
         image = glance.location.ImageProxy(image_stub, context,
                                            self.store_api, self.store_utils)
-        image.set_data('YYYY', None)
-        self.assertEqual(4, image.size)
-        # NOTE(markwash): FakeStore returns image_id for location
-        self.assertEqual(UUID2, image.locations[0]['url'])
-        self.assertEqual('Z', image.checksum)
-        self.assertEqual('active', image.status)
-        image.delete()
-        self.assertEqual(image.status, 'deleted')
-        self.assertRaises(glance_store.NotFound,
-                          self.store_api.get_from_backend,
-                          image.locations[0]['url'], context={})
+
+        if CONF.image_format.require_image_format_match:
+            self.assertRaises(exception.InvalidImageData,
+                              image.set_data, 'YYYY', None)
+        else:
+            image.set_data('YYYY', None)
+            self.assertEqual(4, image.size)
+            # NOTE(markwash): FakeStore returns image_id for location
+            self.assertEqual(UUID2, image.locations[0]['url'])
+            self.assertEqual('Z', image.checksum)
+            self.assertEqual('active', image.status)
+            image.delete()
+            self.assertEqual(image.status, 'deleted')
+            self.assertRaises(glance_store.NotFound,
+                              self.store_api.get_from_backend,
+                              image.locations[0]['url'], context={})
+
+    def test_image_set_data_unknown_size_disabled(self):
+        self.config(require_image_format_match=False, group='image_format')
+        self.test_image_set_data_unknown_size()
 
     @mock.patch('glance.location.LOG')
     def test_image_set_data_valid_signature(self, mock_log):
